<?php

class EAdvancedEmailValidator extends CValidator
{
	
	const CACHE_KEY = 'Extension.AdvancedEmailValidator';
	
	public $cacheID = 'cache';
	public $pattern = '/^[a-z0-9!#$%&\'*+\\/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&\'*+\\/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+(?:{$tlds})\b$/i';
	public $checkMX = false;
	public $checkPort = false;
	public $allowEmpty = true;
	
	protected function validateAttribute($object, $attribute)
	{
		
		$tlds = $this->_getTlds();
		
		$value = $object->$attribute;
		
		if (true === $this->allowEmpty && true === $this->isEmpty($value))
		{
			return;
		}
		
		$valid = (bool)preg_match(str_replace('{$tlds}', $tlds, $this->pattern), $value);

		if (true === $valid)
		{
			
			$domain = rtrim(substr($value, strpos($value,'@') + 1), '>');
			
			if (true === $this->checkMX && true === function_exists('checkdnsrr'))
			{
				$valid = checkdnsrr($domain, 'MX');	
			}
			
			if (true === $valid && true === $this->checkPort && true === function_exists('fsockopen'))
			{
				$valid = (false !== fsockopen($domain, 25));	
			}
		
		}
		
		if (false === $valid)
		{
			$message = (null !== $this->message) ? $this->message : Yii::t('yii', '{attribute} is not a valid email address.');
			$this->addError($object, $attribute, $message);
		}
		
		return $valid;
		
	}

	private function _getTlds()
	{
		
		if (($cache = Yii::app()->getComponent($this->cacheID)) instanceof CCache)
		{
			
			if (false === ($tlds = $cache->get(self::CACHE_KEY)))
			{
				
				$data = @file_get_contents('http://www.iana.org/domains/root/db/');
				
				if (false !== $data)
				{
					
					$tlds = '';
					
					if (false !== (bool)preg_match_all("/<a href=\"\/domains\/root\/db\/([a-z]{2,6}+)\.html\">\.[A-Z]{2,6}+<\/a>/", $data, $matches))
					{
						
						for ($i=0; $i!=count($matches[1]); $i++)
						{
							$tlds .= "{$matches[1][$i]}|";
						}
						
						$tlds = rtrim($tlds, '|');						
						
					}

				}

			}
			
			if (true === empty($tlds))
			{
				$tlds = '[a-z]{2,6}';
				$cache->set(self::CACHE_KEY, $tlds, 3600);
			}
			elseif ('[a-z]{2,6}' !== $tlds)
			{
				$cache->set(self::CACHE_KEY, $tlds, 86400);
			}
			
			return $tlds;
			
		}
		else
		{
			throw new CException("EAdvancedEmailValidator.cacheID \"{$this->cacheID}\" is invalid. Please make sure it refers to the ID of a CCache application component.");
		}
		
	}

}

?>